/*
 * Copyright (C) 2008-2013 Universidade Federal de Pernambuco and 
 * University of Central Florida
 *
 * This file is part of AJML
 *
 * AJML is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * AJML is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AJML; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: QDoxUtil.java,v 1.0 2013/05/17 13:00:38 henriquerebelo Exp $
 */

package org.jmlspecs.util;

import japa.parser.JavaParser;
import japa.parser.ParseException;
import japa.parser.ast.CompilationUnit;
import japa.parser.ast.body.BodyDeclaration;
import japa.parser.ast.body.ConstructorDeclaration;
import japa.parser.ast.body.FieldDeclaration;
import japa.parser.ast.body.MethodDeclaration;
import japa.parser.ast.body.TypeDeclaration;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.jmlspecs.ajmlrac.Main;
import org.multijava.util.compiler.PositionedError;

import com.thoughtworks.qdox.JavaDocBuilder;
import com.thoughtworks.qdox.model.Annotation;
import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaField;
import com.thoughtworks.qdox.model.JavaMethod;
import com.thoughtworks.qdox.model.TypeVariable;


public class QDoxUtil {

	public static String getTypeErasureForTypeDeclsInFile(File file, 
			boolean isGenericSource) throws FileNotFoundException,  IOException, PositionedError, ParseException{
		BufferedReader buffer = null;
		StringBuffer bufferedFile = null;
		String fileAsString = "";
		String line = "";

		bufferedFile = new StringBuffer("");
		buffer = new BufferedReader(new FileReader( file ));
		line = buffer.readLine();
		while (line != null) {
			bufferedFile.append(line);
			bufferedFile.append("\n");
			line = buffer.readLine();
		}
		buffer.close();
		fileAsString = bufferedFile.toString();
	
		AspectUtil.getInstance().addProcessedJavaFile(fileAsString);
		JavaDocBuilder qDoxFile = new JavaDocBuilder();
		qDoxFile.addSource(new FileReader(file));
		
		// handling JavaDocTags in File
		List<DocletTag> javaMethsWithDocletTagsFile = QDoxUtil.getAllJavaDocTagsInFile(qDoxFile);
		if(javaMethsWithDocletTagsFile.size() > 0){
			buffer = new BufferedReader(new StringReader(fileAsString));
			StringBuffer fileAsStringJavaDocProcessed = new StringBuffer("");
			line = buffer.readLine();
			int fileLineNumber = 1;
			while (line != null) {
				// if any
				String jmlClause = QDoxUtil.getJavaDocTagAsJMLClause(javaMethsWithDocletTagsFile, fileLineNumber);
				fileAsStringJavaDocProcessed.append(line).append(jmlClause);
				fileAsStringJavaDocProcessed.append("\n");
				line = buffer.readLine();
				fileLineNumber++;
			}
			buffer.close();
			fileAsString = StringUtils.replaceOnce(fileAsString, fileAsString, fileAsStringJavaDocProcessed.toString());
		}
		
		// handling javadoc tags in Java types that should be shifted
		List<JavaClass> javaDeclTypeWithJavadocTags = QDoxUtil.getAllDeclaredJavaTypesWithJavaDocTagsInFile(qDoxFile);
		for (Iterator<JavaClass> iterator = javaDeclTypeWithJavadocTags.iterator(); iterator
				.hasNext();) {
			JavaClass javaTypeWithJavadoc = iterator.next();
			String jmlClausesToShift = QDoxUtil.getJavaDocTagAsJMLClauseForTypeToShift(QDoxUtil.getAllJavaDocTagsInAJavaTypedDecl(javaTypeWithJavadoc));
			if(jmlClausesToShift.equals("")){
				continue;
			}
			buffer = new BufferedReader(new StringReader(fileAsString));
			int lineStart = javaTypeWithJavadoc.getLineNumber();
			int fileLineNumber = 1;
			StringBuffer TypeDeclUtilOpenBrace = new StringBuffer("");
			line = buffer.readLine();
			fileLineNumber = 1;
			while (line != null) {
				if(fileLineNumber >= lineStart){
					if(line.contains("{")){
						int indexOpenBrace = line.indexOf('{');
						String lineTmp = line.substring(0, (indexOpenBrace+1));
						TypeDeclUtilOpenBrace.append(lineTmp);
						break;
					}
					else{
						TypeDeclUtilOpenBrace.append(line);
						TypeDeclUtilOpenBrace.append("\n");
					}
				}
				line = buffer.readLine();
				fileLineNumber++;
			}
			buffer.close();
			String TypeDeclUtilOpenBraceStr = TypeDeclUtilOpenBrace.toString().trim();
			// processing java field tags
			// placing them where the JML compiler can understand - [[[hemr]]]
			fileAsString = StringUtils.replaceOnce(fileAsString, TypeDeclUtilOpenBraceStr, TypeDeclUtilOpenBraceStr+jmlClausesToShift);
		}// end for
		
		// handling javadoc tags in Java fields that should be shifted
		List<JavaField> javaDeclFieldsWithJavadocTags = QDoxUtil.getAllDeclaredJavaFieldsWithJavaDocTagsInFile(qDoxFile);
		for (Iterator<JavaField> iterator = javaDeclFieldsWithJavadocTags.iterator(); iterator
				.hasNext();) {
			JavaField javaFieldWithJavadoc = iterator.next();
			String jmlClausesToShift = QDoxUtil.getJavaDocTagAsJMLClauseForFieldToShift(QDoxUtil.getAllJavaDocTagsInAJavaFieldDecl(javaFieldWithJavadoc));
			if(jmlClausesToShift.equals("")){
				continue;
			}
			buffer = new BufferedReader(new StringReader(fileAsString));
			int lineStart = javaFieldWithJavadoc.getLineNumber();
			int fileLineNumber = 1;
			StringBuffer fieldDecl = new StringBuffer("");
			line = buffer.readLine();
			fileLineNumber = 1;
			while (line != null) {
				if(fileLineNumber >= lineStart){
					if(line.contains(";")){
						int indexSemiColon = line.lastIndexOf(';');
						fieldDecl.append(line.substring(0, indexSemiColon+1));
						break;
					}
					else{
						fieldDecl.append(line);
						fieldDecl.append("\n");
					}
				}
				line = buffer.readLine();
				fileLineNumber++;
			}
			buffer.close();
			String fieldDeclStr = fieldDecl.toString().trim();
			// processing java field tags
			// placing them where the JML compiler can understand - [[[hemr]]]
			fileAsString = StringUtils.replaceOnce(fileAsString, fieldDeclStr, fieldDeclStr+jmlClausesToShift);
		}// end for
		
		// Generic Source or any Java 5+ features (hopefully :-)
		if(isGenericSource){
			bufferedFile.delete(0, (bufferedFile.length()-1)); // reset for later use

			// handling enum types
			List<JavaClass> javaDeclEnumTypes = QDoxUtil.getAllDeclaredJavaEnumTypesInFile(qDoxFile);
			if(javaDeclEnumTypes.size() > 0){
				fileAsString = QDoxUtil.getFileEnumTypeErasureProcessingAsString(bufferedFile, fileAsString, javaDeclEnumTypes);
			}
			
			// collecting all methods that lexically occur within a file
			bufferedFile.delete(0, (bufferedFile.length()-1)); // reset for later use
			List<JavaMethod> javaDeclMeths = QDoxUtil.getAllDeclaredJavaMethodsInFile(qDoxFile);
			List<String> fileMeths = QDoxUtil.getAllJavaMethodDeclLexicallyInFile(bufferedFile, fileAsString, javaDeclMeths);
			
			if(fileMeths.size() != javaDeclMeths.size()){
				System.out.println("file = "+file.getCanonicalPath());
				System.out.println("processed ---> "+fileMeths.size());
				System.out.println("really contains = "+javaDeclMeths.size());
			}
			
//			for (Iterator<String> iterator = fileMeths.iterator(); iterator.hasNext();) {
//			String currentMeth = iterator.next();
//			System.out.println("matchedMeth = "+currentMeth);
//		    }

			fileAsString = QDoxUtil.stripMethBodies(fileAsString, javaDeclMeths, fileMeths); // method bodies stripped... [[[hemr]]]
			
//			BufferedReader buffer2 = new BufferedReader(new StringReader(fileAsString));
//			line = buffer2.readLine();
//			int count = 1;
//			while (line != null) {
//				System.out.println(count+": "+line);
//				line = buffer2.readLine();
//				count++;
//			}
//			buffer2.close();
			
			
//			System.out.println(fileAsString);

			// handling annotated Java fields
			List<JavaField> javaDeclAnnotatedFields = QDoxUtil.getAllDeclaredAnnotatedJavaFieldsInFile(qDoxFile);
			for (Iterator<JavaField> iterator = javaDeclAnnotatedFields.iterator(); iterator
					.hasNext();) {
				JavaField annotatedJavaField = iterator.next();
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer annotationArea = new StringBuffer("");
				StringBuffer annotationAreaCommented = new StringBuffer("");
				int lineStart = annotatedJavaField.getAnnotations()[0].getLineNumber();
				int lineEnd = QDoxUtil.getFieldAnnotationLineEndWithJavaParser(lineStart, fileAsString);
				line = buffer.readLine();
				int fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber >= lineStart){
						if(fileLineNumber == lineEnd){
							annotationArea.append(line);
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line.replace("@", "#")+"*/");
							break;
						}
						else{
							annotationArea.append(line);
							annotationArea.append("\n");
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line.replace("@", "#")+"*/");
							annotationAreaCommented.append("\n");
						}
					}				
					line = buffer.readLine();
					fileLineNumber++;
				} // end while
				buffer.close();
				
				// pre field annotations if any
				StringBuffer fieldDeclAnnotations = new StringBuffer("");
				if(annotationArea.toString().contains("@SpecPublic")){
					fieldDeclAnnotations.append("spec_public ");
				}
				if(annotationArea.toString().contains("@SpecProtected")){
					fieldDeclAnnotations.append("spec_protected ");
				}
				if(annotationArea.toString().contains("@NonNull")){
					fieldDeclAnnotations.append("non_null ");
				}
				if(annotationArea.toString().contains("@Nullable")){
					fieldDeclAnnotations.append("nullable ");
				}
				if(annotationArea.toString().contains("@Model")){
					fieldDeclAnnotations.append("model ");
				}
				
				// processing java field annotations
				String annotationAreaCommentedStr = QDoxUtil.getFieldAnnotationAreaCommentedProcessedWithJML(
						annotatedJavaField, annotationAreaCommented.toString());

				// doing the replacement
				int newLineEnd = QDoxUtil.getFieldLineEndWithJavaParser(lineStart, fileAsString);
				lineStart = lineEnd;
				lineEnd = newLineEnd;
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer fileAsStringReplacement = new StringBuffer("");
				line = buffer.readLine();
				fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber == (lineStart + 1)){
						// placing them where the JML compiler can understand - [[[hemr]]]
						fileAsStringReplacement.append("/*@  "+fieldDeclAnnotations.toString()+"@*/"+" "+line);
						fileAsStringReplacement.append("\n");
					}
					else if(fileLineNumber == (lineEnd)){ // getting the end of field line
						// placing them where the JML compiler can understand - [[[hemr]]]
						fileAsStringReplacement.append(line+annotationAreaCommentedStr);
						fileAsStringReplacement.append("\n");
					}
					else{
						fileAsStringReplacement.append(line);
						fileAsStringReplacement.append("\n");
					}
					line = buffer.readLine();
					fileLineNumber++;
				} // end while
				buffer.close();
				
				fileAsString = fileAsStringReplacement.toString();
				// updating the current field annotations in the file
				// removing field annotations
				fileAsString = StringUtils.replaceOnce(fileAsString, annotationArea.toString(), "");
			}// end for
			
			// handling annotated Java Methods
			List<String> javaDeclMethsAnnotationArea = new ArrayList<String>();
			List<JavaMethod> javaDeclAnnotatedMeths = QDoxUtil.getAllDeclaredAnnotatedJavaMethodsInFile(qDoxFile);
			for (Iterator<JavaMethod> iterator = javaDeclAnnotatedMeths.iterator(); iterator
					.hasNext();) {
				JavaMethod annotatedJavaMethod = iterator.next();
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer annotationArea = new StringBuffer("");
				StringBuffer annotationAreaCommented = new StringBuffer("");
				int lineStart = annotatedJavaMethod.getLineNumber();
				int lineEnd = QDoxUtil.getMethodAnnotationLineEndWithJavaParser(lineStart, fileAsString);
				line = buffer.readLine();
				int fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber >= lineStart){
						if(fileLineNumber == lineEnd){
							annotationArea.append(line);
							annotationArea.append("\n");
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line+"*/");
							annotationAreaCommented.append("\n");
							break;
						}
						else{
							annotationArea.append(line);
							annotationArea.append("\n");
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line+"*/");
							annotationAreaCommented.append("\n");
						}
					}
					line = buffer.readLine();
					fileLineNumber++;
				} // end while

				// processing java method annotations
				String annotationAreaCommentedStr = QDoxUtil.getMethodAnnotationAreaCommentedProcessedWithJML(
						annotatedJavaMethod, annotationAreaCommented);
				
				// updating the current meth annotations in the file
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer fileStartCurrentMeth = new StringBuffer("");
				
				lineStart = annotatedJavaMethod.getLineNumber();
				lineEnd = QDoxUtil.getMethodLineEndWithJavaParser(lineStart, fileAsString);
				line = buffer.readLine();
				fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber >= lineStart){
						if(fileLineNumber == lineEnd){
							fileStartCurrentMeth.append(line);
							fileStartCurrentMeth.append("\n");
							break;
						}
						else{
							fileStartCurrentMeth.append(line);
							fileStartCurrentMeth.append("\n");
						}
						
					}
					line = buffer.readLine();
					fileLineNumber++;
				}
				String fileAsStringTmp = StringUtils.replaceOnce(fileStartCurrentMeth.toString(), annotationArea.toString(), annotationAreaCommentedStr.toString());
				fileAsString = StringUtils.replaceOnce(fileAsString, fileStartCurrentMeth.toString(), fileAsStringTmp);
				javaDeclMethsAnnotationArea.add(annotationAreaCommentedStr);
			}// end for
			
			// handling annotated Java Types
			List<JavaClass> javaDeclAnnotatedTypes = QDoxUtil.getAllDeclaredAnnotatedJavaTypesInFile(qDoxFile);
			for (Iterator<JavaClass> iterator = javaDeclAnnotatedTypes.iterator(); iterator
					.hasNext();) {
				JavaClass annotatedJavaType = iterator.next();
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer annotationArea = new StringBuffer("");
				StringBuffer annotationAreaCommented = new StringBuffer("");
				int lineStart = annotatedJavaType.getAnnotations()[0].getLineNumber();
				int lineEnd = QDoxUtil.getTypeAnnotationLineEndWithJavaParser(lineStart, fileAsString);
				if(lineEnd == 0){
//					System.out.println("lineStart<1> = "+lineStart);
					lineStart = (annotatedJavaType.getAnnotations()[0].getLineNumber() - 1);
					lineEnd = QDoxUtil.getTypeAnnotationLineEndWithJavaParser(lineStart, fileAsString);
				}
//				System.out.println("lineStart = "+lineStart);
//				System.out.println("lineEnd = "+lineEnd);
				line = buffer.readLine();
				int fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber >= lineStart){
						if(fileLineNumber == lineEnd){
							annotationArea.append(line);
							annotationArea.append("\n");
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line.replace("@", "#")+"*/");
							annotationAreaCommented.append("\n");
							break;
						}
						else{
							annotationArea.append(line);
							annotationArea.append("\n");
							annotationAreaCommented.append("/*"+fileLineNumber+"*/"+"/* "+line.replace("@", "#")+"*/");
							annotationAreaCommented.append("\n");
						}
					}
					line = buffer.readLine();
					fileLineNumber++;
				} // end while
				
				// selecting the entire type decl
				buffer = new BufferedReader(new StringReader(fileAsString));
				StringBuffer typeDecl = new StringBuffer("");
                int newEnd = QDoxUtil.getTypeLineEndWithJavaParser(lineStart, fileAsString);
				lineStart = lineEnd + 1;
				lineEnd = newEnd;
				line = buffer.readLine();
				fileLineNumber = 1;
				while (line != null) {
					if(fileLineNumber >= lineStart){
						if(fileLineNumber == lineEnd){
							typeDecl.append(line);
							typeDecl.append("\n");
						}
						else{
							typeDecl.append(line);
							typeDecl.append("\n");
						}
					}
					line = buffer.readLine();
					fileLineNumber++;
				}
				
				String typeJMLAnno = "";
				if(annotationArea.toString().contains("@NonNullByDefault")){
					typeJMLAnno = "/*@ non_null_by_default @*/";
				}
				if(annotationArea.toString().contains("@NullableByDefault")){
					typeJMLAnno = "/*@ nullable_by_default @*/" + typeJMLAnno;		
				}
				if(annotationArea.toString().contains("@SpecPublic")){
					typeJMLAnno = "/*@ spec_public @*/" + typeJMLAnno;
				}
				if(annotationArea.toString().contains("@SpecProtected")){
					typeJMLAnno = "/*@ spec_protected @*/" + typeJMLAnno;		
				}
				
				// processing java type annotations
				String annotationAreaCommentedStr = QDoxUtil.getTypeAnnotationAreaCommentedProcessedWithJML(
						annotatedJavaType, annotationAreaCommented.toString());
				
				// placing them where the JML compiler can understand - [[[hemr]]]
				String typeDeclStr = StringUtils.replaceOnce(typeDecl.toString(), "{", "{"+annotationAreaCommentedStr);
				
				// updating the current type annotations in the file
				// removing type annotations
				fileAsString = StringUtils.replaceOnce(fileAsString, annotationArea.toString(), typeJMLAnno);
				fileAsString = StringUtils.replaceOnce(fileAsString, typeDecl.toString(), typeDeclStr);
			}// end for
			
//			// collecting all methods that lexically occur within a file
			bufferedFile.delete(0, (bufferedFile.length()-1)); // reset for later use
			javaDeclMeths = QDoxUtil.getAllDeclaredJavaMethodsInFile(qDoxFile);
			fileMeths = QDoxUtil.getAllJavaMethodDeclLexicallyInFile(bufferedFile, fileAsString, javaDeclMeths, javaDeclMethsAnnotationArea);

//			for (Iterator<String> iterator = fileMeths.iterator(); iterator.hasNext();) {
//				String currentMeth = iterator.next();
//				System.out.println("matchedMeth = "+currentMeth);
//			}

			fileAsString = QDoxUtil.getFileMethDeclsProcessed(fileAsString, javaDeclMeths, fileMeths, file);
			
			// final issues about JML type annotations
			Pattern jmlAnnoPattern = Pattern.compile("@(\\s)*Pure(\\b)(\\s)*(\\((\\s)*\\))?");
			Matcher jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ pure @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			jmlAnnoMatcher.reset();
			jmlAnnoPattern = Pattern.compile("@(\\s)*Helper(\\b)(\\s)*(\\((\\s)*\\))?");
			jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ helper @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			jmlAnnoMatcher.reset();
			jmlAnnoPattern = Pattern.compile("@(\\s)*Nullable(\\b)(\\s)*(\\((\\s)*\\))?");
			jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ nullable @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			jmlAnnoMatcher.reset();
			jmlAnnoPattern = Pattern.compile("@(\\s)*NonNull(\\b)(\\s)*(\\((\\s)*\\))?");
			jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ non_null @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			jmlAnnoMatcher.reset();
			jmlAnnoPattern = Pattern.compile("@(\\s)*SpecPublic(\\b)(\\s)*(\\((\\s)*\\))?");
			jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ spec_public @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			jmlAnnoMatcher.reset();
			jmlAnnoPattern = Pattern.compile("@(\\s)*SpecProtected(\\b)(\\s)*(\\((\\s)*\\))?");
			jmlAnnoMatcher = jmlAnnoPattern.matcher(fileAsString);
			while(jmlAnnoMatcher.find()){
				int numberOfNewLines = QDoxUtil.getLineNumbersQtd(jmlAnnoMatcher.group());
				fileAsString = StringUtils.replaceOnce(fileAsString, jmlAnnoMatcher.group(), "/*@ spec_protected @*/"+QDoxUtil.getNewLinesCaracter(numberOfNewLines));
			}
			fileAsString = fileAsString.replaceAll("/\\*(\\s)*/\\*@", "/*@");
			fileAsString = fileAsString.replaceAll("@\\*/(\\s)*\\*/", "@*/");
			// final issues due to annotation types support
			fileAsString = fileAsString.replace("@interface", "interface");
//			fileAsString = fileAsString.replaceAll("\"[\\w\"\\s+\\{\\}\\(\\)\\.]*(\\b)default(\\b)[\\w\"\\s+\\{\\}\\(\\)\\.]*\"", "\"\""); // improve! [[[hemr]]] **
			
			Pattern defaultTextPattern = Pattern.compile("\"[\\w\"\\s+\\{\\}\\(\\)\\.]*(\\b)default(\\b)[\\w\"\\s+\\{\\}\\(\\)\\.]*\"");
			Matcher defaultTextMatcher = defaultTextPattern.matcher(fileAsString);
			while(defaultTextMatcher.find()){
				fileAsString = StringUtils.replaceOnce(fileAsString, defaultTextMatcher.group(), defaultTextMatcher.group().replace("default", ""));
			}
			
			// handling default stmt in annotation types
			fileAsString = fileAsString.replaceAll("(\\b)default(\\b)(\\s)*[\\w\"\\s+\\{\\}\\(\\)\\.]*(\\s)*;", ";");
			
			// handling static imports to support java 5 mode
			Pattern staticImportPattern = Pattern.compile("import static (.)*;");
			Matcher staticImportMatcher = staticImportPattern.matcher(fileAsString);
			while(staticImportMatcher.find()){
				String staticImport = staticImportMatcher.group();
				staticImport = staticImport.substring(0, staticImport.lastIndexOf(".")).replace("static ", "")+";";
				fileAsString = StringUtils.replaceOnce(fileAsString, staticImportMatcher.group(), staticImport);
			}
		} // final generic treatment
		
		// for debuggin purposes remove the following comment - [[[hemr]]]
//		System.err.println(fileAsString);
		return fileAsString;
	}
	
	private static String getJavaDocTagAsJMLClauseForTypeToShift(List<DocletTag> docletTagFieldsToShift){
		StringBuffer jmlClauses = new StringBuffer("");
		for (Iterator<DocletTag> iterator = docletTagFieldsToShift.iterator(); iterator.hasNext();) {
			DocletTag docletTag = iterator.next();
			String tagName = docletTag.getName();
			String tagValue = docletTag.getValue().replaceAll("\n", "").replaceAll("\r", "");
			if(tagName.equals("model_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("model_method_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("model_field_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("invariant_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("constraint_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("initially_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
			else if(tagName.equals("represents_definition")){
				String clause = "/*@ "+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
		}
		return jmlClauses.toString();
	}
	
	private static String getJavaDocTagAsJMLClauseForFieldToShift(List<DocletTag> docletTagFieldsToShift){
		StringBuffer jmlClauses = new StringBuffer("");
		for (Iterator<DocletTag> iterator = docletTagFieldsToShift.iterator(); iterator.hasNext();) {
			DocletTag docletTag = iterator.next();
			String tagName = docletTag.getName();
			String tagValue = docletTag.getValue().replaceAll("\n", "").replaceAll("\r", "");
			if(tagName.equals("in")){
				String clauseKind = "in";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				jmlClauses.append(clause);
			}
		}
		return jmlClauses.toString();
	}
	
	private static String getJavaDocTagAsJMLClause(List<DocletTag> docletTags, int fileLineNumber){
		DocletTag foundTag = null;
		String result = "";
		for (Iterator<DocletTag> iterator = docletTags.iterator(); iterator.hasNext();) {
			DocletTag docletTag = iterator.next();
			if(docletTag.getLineNumber() == fileLineNumber){
				foundTag = docletTag;
			}
		}
		if(foundTag != null){
//			*/   /*10*/  /*@@*/  /** sample 
			String tagName = foundTag.getName();
			String tagValue = foundTag.getValue().replaceAll("\n", "").replaceAll("\r", "");
			if(tagName.equals("nullable_by_default")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "nullable_by_default";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("non_null_by_default")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "non_null_by_default";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("spec_case_header")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clause = "/*@ "+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("helper")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "helper";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("pure")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "pure";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("also")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "also";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("spec_public")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "spec_public";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("spec_protected")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "spec_protected";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("non_null")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "non_null";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("nullable")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "nullable";
				String clause = "/*@ "+clauseKind+" "+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("requires")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "requires";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("ensures")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "ensures";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("assignable")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "assignable";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("signals")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "signals";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("signals_only")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "signals_only";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("callable")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "callable";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("accessible")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "accessible";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("duration")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "duration";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("captures")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "captures";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
			else if(tagName.equals("working_space")){
				String start = "*/";
				String end = "/**";
				String line = "/*"+foundTag.getLineNumber()+"*/";
				String clauseKind = "working_space";
				String clause = "/*@ "+clauseKind+" "+tagValue+"@*/";
				result = start+line+clause+end;
			}
		}
		return result;
	}
	
	private static String getFieldAnnotationAreaCommentedProcessedWithJML(
			JavaField annotatedJavaField, String annotationAreaCommentedStr) {
		//start
		for (int i = 0; i < annotatedJavaField.getAnnotations().length; i++) {
			Annotation currentAnnotation = annotatedJavaField.getAnnotations()[i];
			// handling single JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.In")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "in_redundantly" : "in") : "in");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			// handling composite JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.InDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "in_redundantly" : "in") : "in");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}
		}
		return annotationAreaCommentedStr;
	}

	private static String getTypeAnnotationAreaCommentedProcessedWithJML(
			JavaClass annotatedJavaType, String annotationAreaCommentedStr) {
		//start
		for (int i = 0; i < annotatedJavaType.getAnnotations().length; i++) {
			Annotation currentAnnotation = annotatedJavaType.getAnnotations()[i];
			// handling single JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Invariant")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = (currentAnnotation.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentAnnotation.getProperty("header").toString())+" ":"");
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "invariant_redundantly" : "invariant") : "invariant");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Constraint")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = (currentAnnotation.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentAnnotation.getProperty("header").toString())+" ":"");
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "constraint_redundantly" : "constraint") : "constraint");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Initially")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = (currentAnnotation.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentAnnotation.getProperty("header").toString())+" ":"");
					String clauseKind = "initially";
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Represents")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = (currentAnnotation.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentAnnotation.getProperty("header").toString())+" ":"");
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "represents_redundantly" : "represents") : "represents");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Model")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = "model ";
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.ModelMethod")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = "model ";
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.ModelField")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String header = "model ";
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+header+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			// handling composite JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.InvariantDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = (currentDef.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentDef.getProperty("header").toString())+" ":"");
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "invariant_redundantly" : "invariant") : "invariant");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.ConstraintDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = (currentDef.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentDef.getProperty("header").toString())+" ":"");
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "constraint_redundantly" : "constraint") : "constraint");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.InitiallyDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = (currentDef.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentDef.getProperty("header").toString())+" ":"");
							String clauseKind = "initially";
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.RepresentsDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = (currentDef.getNamedParameterMap().containsKey("header")? QDoxUtil.preparePred(currentDef.getProperty("header").toString())+" ":"");
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "represents_redundantly" : "represents") : "represents");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.ModelDefinitions")){
				if(currentAnnotation.getPropertyMap().containsKey("value")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = "model ";
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}	
				}
				if(currentAnnotation.getPropertyMap().containsKey("fields")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("fields").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = "model ";
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}	
				}
				if(currentAnnotation.getPropertyMap().containsKey("methods")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("methods").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String header = "model ";
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+header+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}	
				}
			}
		}
		// end
		return annotationAreaCommentedStr;
	}

	// helper methods

	private static List<JavaMethod> getAllDeclaredJavaMethodsInFile(JavaDocBuilder qDoxFile){
		List<JavaMethod> result = new ArrayList<JavaMethod>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			JavaMethod [] javaDeclMethods = currentJavaDeclType.getMethods();
			for (int j = 0; j < javaDeclMethods.length; j++) {
				JavaMethod currentJavaDeclMethod = javaDeclMethods[j];
				result.add(currentJavaDeclMethod);
			}
		}
		Comparator<JavaMethod> COMPARE_BY_LINE_NUMBER = new Comparator<JavaMethod>() {
			public int compare(JavaMethod one, JavaMethod other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<JavaClass> getAllDeclaredAnnotatedJavaTypesInFile(JavaDocBuilder qDoxFile){
		List<JavaClass> result = new ArrayList<JavaClass>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			if(currentJavaDeclType.getAnnotations().length > 0){
				result.add(currentJavaDeclType);
			}
		}
		Comparator<JavaClass> COMPARE_BY_LINE_NUMBER = new Comparator<JavaClass>() {
			public int compare(JavaClass one, JavaClass other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<JavaClass> getAllDeclaredJavaTypesWithJavaDocTagsInFile(JavaDocBuilder qDoxFile){
		List<JavaClass> result = new ArrayList<JavaClass>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			if(currentJavaDeclType.getTags().length > 0){
				result.add(currentJavaDeclType);
			}
		}
		Comparator<JavaClass> COMPARE_BY_LINE_NUMBER = new Comparator<JavaClass>() {
			public int compare(JavaClass one, JavaClass other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}

	private static List<JavaField> getAllDeclaredAnnotatedJavaFieldsInFile(JavaDocBuilder qDoxFile){
		List<JavaField> result = new ArrayList<JavaField>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			JavaField [] javaDeclFields = currentJavaDeclType.getFields();
			for (int j = 0; j < javaDeclFields.length; j++) {
				JavaField currentJavaDeclField = javaDeclFields[j];
				if(currentJavaDeclField.getAnnotations().length > 0){
					result.add(currentJavaDeclField);
				}
			}
		}
		Comparator<JavaField> COMPARE_BY_LINE_NUMBER = new Comparator<JavaField>() {
			public int compare(JavaField one, JavaField other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<JavaField> getAllDeclaredJavaFieldsWithJavaDocTagsInFile(JavaDocBuilder qDoxFile){
		List<JavaField> result = new ArrayList<JavaField>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			JavaField [] javaDeclFields = currentJavaDeclType.getFields();
			for (int j = 0; j < javaDeclFields.length; j++) {
				JavaField currentJavaDeclField = javaDeclFields[j];
				if(currentJavaDeclField.getTags().length > 0){
					result.add(currentJavaDeclField);
				}
			}
		}
		Comparator<JavaField> COMPARE_BY_LINE_NUMBER = new Comparator<JavaField>() {
			public int compare(JavaField one, JavaField other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<JavaMethod> getAllDeclaredAnnotatedJavaMethodsInFile(JavaDocBuilder qDoxFile){
		List<JavaMethod> result = new ArrayList<JavaMethod>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			JavaMethod [] javaDeclMethods = currentJavaDeclType.getMethods();
			for (int j = 0; j < javaDeclMethods.length; j++) {
				JavaMethod currentJavaDeclMethod = javaDeclMethods[j];
				if(currentJavaDeclMethod.getAnnotations().length > 0){
					result.add(currentJavaDeclMethod);
				}
			}
		}
		Comparator<JavaMethod> COMPARE_BY_LINE_NUMBER = new Comparator<JavaMethod>() {
			public int compare(JavaMethod one, JavaMethod other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<DocletTag> getAllJavaDocTagsInAJavaTypedDecl(JavaClass type){
		List<DocletTag> result = new ArrayList<DocletTag>();
		if(type.getTags().length > 0){
			for (int i = 0; i < type.getTags().length; i++) {
				result.add(type.getTags()[i]);
			}
		}
		Comparator<DocletTag> COMPARE_BY_LINE_NUMBER = new Comparator<DocletTag>() {
			public int compare(DocletTag one, DocletTag other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<DocletTag> getAllJavaDocTagsInAJavaFieldDecl(JavaField field){
		List<DocletTag> result = new ArrayList<DocletTag>();
		if(field.getTags().length > 0){
			for (int i = 0; i < field.getTags().length; i++) {
				result.add(field.getTags()[i]);
			}
		}
		Comparator<DocletTag> COMPARE_BY_LINE_NUMBER = new Comparator<DocletTag>() {
			public int compare(DocletTag one, DocletTag other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}
	
	private static List<DocletTag> getAllJavaDocTagsInFile(JavaDocBuilder qDoxFile){
		List<DocletTag> result = new ArrayList<DocletTag>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			
			// type javadoc tags
			if(currentJavaDeclType.getTags().length > 0){
				for (int k = 0; k < currentJavaDeclType.getTags().length; k++) {
					result.add(currentJavaDeclType.getTags()[k]);
				}
			}
			// field javadoc tags
			JavaField [] javaDeclFields = currentJavaDeclType.getFields();
			for (int j = 0; j < javaDeclFields.length; j++) {
				JavaField currentJavaDeclField = javaDeclFields[j];
				if(currentJavaDeclField.getTags().length > 0){
					for (int k = 0; k < currentJavaDeclField.getTags().length; k++) {
						result.add(currentJavaDeclField.getTags()[k]);
					}
				}
			}
			// method javadoc tags
			JavaMethod [] javaDeclMethods = currentJavaDeclType.getMethods();
			for (int j = 0; j < javaDeclMethods.length; j++) {
				JavaMethod currentJavaDeclMethod = javaDeclMethods[j];
				if(currentJavaDeclMethod.getTags().length > 0){
					for (int k = 0; k < currentJavaDeclMethod.getTags().length; k++) {
						result.add(currentJavaDeclMethod.getTags()[k]);
					}
				}
			}
		}
		Comparator<DocletTag> COMPARE_BY_LINE_NUMBER = new Comparator<DocletTag>() {
			public int compare(DocletTag one, DocletTag other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}

	private static List<JavaField> getADeclaredJavaEnumerationFieldsInEnumType(JavaClass enumType){
		List<JavaField> result = new ArrayList<JavaField>();

		JavaField [] javaDeclFields = enumType.getFields();
		for (int j = 0; j < javaDeclFields.length; j++) {
			JavaField currentJavaDeclField = javaDeclFields[j];
			if(enumType.getLineNumber() == currentJavaDeclField.getLineNumber()){
				result.add(currentJavaDeclField);
			}
		}
		Comparator<JavaField> COMPARE_BY_LINE_NUMBER = new Comparator<JavaField>() {
			public int compare(JavaField one, JavaField other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}

	private static String getADeclaredJavaEnumerationFieldsInEnumTypeAsErasure(JavaClass enumType){
		List<JavaField> javaEnumFields = new ArrayList<JavaField>();
		StringBuffer code = new StringBuffer("");

		JavaField [] javaDeclFields = enumType.getFields();
		for (int j = 0; j < javaDeclFields.length; j++) {
			JavaField currentJavaDeclField = javaDeclFields[j];
			if(enumType.getLineNumber() == currentJavaDeclField.getLineNumber()){
				javaEnumFields.add(currentJavaDeclField);
			}
		}
		Comparator<JavaField> COMPARE_BY_LINE_NUMBER = new Comparator<JavaField>() {
			public int compare(JavaField one, JavaField other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(javaEnumFields, COMPARE_BY_LINE_NUMBER);

		for (Iterator<JavaField> iterator = javaEnumFields.iterator(); iterator.hasNext();) {
			JavaField javaEnumField = iterator.next();
			code.append("public static final "+javaEnumField.getType().getFullyQualifiedName()+" "+javaEnumField.getName()+" = null;");
		}

		return code.toString();
	}

	private static List<JavaClass> getAllDeclaredJavaEnumTypesInFile(JavaDocBuilder qDoxFile){
		List<JavaClass> result = new ArrayList<JavaClass>();
		JavaClass [] javaDeclTypes  = qDoxFile.getClasses();

		for (int i = 0; i < javaDeclTypes.length; i++) {
			JavaClass currentJavaDeclType = javaDeclTypes[i];
			if(currentJavaDeclType.isEnum()){
				result.add(currentJavaDeclType);
			}
		}
		Comparator<JavaClass> COMPARE_BY_LINE_NUMBER = new Comparator<JavaClass>() {
			public int compare(JavaClass one, JavaClass other) {
				return Integer.valueOf(one.getLineNumber()).compareTo(other.getLineNumber());
			}
		};
		Collections.sort(result, COMPARE_BY_LINE_NUMBER);
		return result;
	}

	private static HashMap<String, String> getGenericAndErasureTypeParametersForMethod(JavaClass clazz, JavaMethod meth){
		HashMap<String, String> result = new HashMap<String, String>();
		// adding the current type generic type parameters
		if(clazz.getTypeParameters().length > 0){
			TypeVariable [] typeVariables = clazz.getTypeParameters();
			for (int i = 0; i < typeVariables.length; i++) {
				TypeVariable currentTypeVariable = typeVariables[i];
				String genericType = currentTypeVariable.getName();
				String erasureType = currentTypeVariable.getValue();
				if(erasureType.equals("")){
					erasureType = "java.lang.Object";
				}
				result.put(genericType, erasureType);
			}
		}
		// adding the current method generic type parameters
		if(meth.getTypeParameters().length > 0){
			TypeVariable [] typeVariables = meth.getTypeParameters();
			for (int i = 0; i < typeVariables.length; i++) {
				TypeVariable currentTypeVariable = typeVariables[i];
				String genericType = currentTypeVariable.getName();
				String erasureType = currentTypeVariable.getValue();
				if(erasureType.equals("")){
					erasureType = "java.lang.Object";
				}
				result.put(genericType, erasureType);
			}
		}
		return result;
	}

	private static String stripMethBodies(
			String fileAsString, List<JavaMethod> javaDeclMeths,
			List<String> fileMeths) {
		int indexMeth = 0;
		for (Iterator<String> iterator = fileMeths.iterator(); iterator.hasNext();) {
			String returnTypeDefaultValue = "";
			String currentMeth = iterator.next();
			String currentMethStrippedBody = "";
			currentMethStrippedBody = currentMeth;
			JavaMethod javaMeth = javaDeclMeths.get(indexMeth);
			
			if(((javaMeth.getReturnType() != null) && !(javaMeth.isConstructor()))){
				if(!QDoxUtil.defaultValue(javaMeth.getReturnType().getFullyQualifiedName()).equals("")){
					returnTypeDefaultValue = "return "+QDoxUtil.defaultValue(javaMeth.getReturnType().getFullyQualifiedName())+";";
				}
			}
			
			if(!currentMeth.equals("")){
				int openBraceIndex = currentMeth.indexOf('{');
				if(openBraceIndex != -1){
					String tmp = "";
					StringBuffer linesToAppend = new StringBuffer("");
					tmp = currentMeth.substring(openBraceIndex+1);
					int newLineCount = QDoxUtil.getLineNumbersQtd(tmp);
					linesToAppend.append("{");
					linesToAppend.append(QDoxUtil.getNewLinesCaracter(newLineCount));
					linesToAppend.append(returnTypeDefaultValue).append("}");
//					currentMethStrippedBody = currentMeth.replaceAll("\\{[\\w\\W]*\\}", linesToAppend.toString());
					String body = currentMeth.substring(openBraceIndex, currentMeth.lastIndexOf("}")+1);
					currentMethStrippedBody = currentMeth.replace(body, linesToAppend.toString());
					fileAsString = StringUtils.replaceOnce(fileAsString, currentMeth, currentMethStrippedBody);
				}
			}
			indexMeth++;
		}
		return fileAsString;
	}
	
	private static String getFileMethDeclsProcessed(
			String fileAsString, List<JavaMethod> javaDeclMeths,
			List<String> fileMeths, File file) throws PositionedError {
		int indexMeth = 0;
		for (Iterator<String> iterator = fileMeths.iterator(); iterator.hasNext();) {
			String currentMethErasure = "";
			String currentMethErasureTmp = "";
			String currentMeth = iterator.next();
			JavaMethod javaMeth = javaDeclMeths.get(indexMeth);
			if(javaMeth.getAnnotations().length > 0){
				// processing method annotations including parameter ones
				String currentMethAnnoProcessed = QDoxUtil.removeNonJMLAnnotationParameters(currentMeth);
				fileAsString = StringUtils.replaceOnce(fileAsString, currentMeth, currentMethAnnoProcessed);
				currentMeth = currentMethAnnoProcessed;
			}
			currentMethErasureTmp = currentMeth;
			JavaClass onwerType = javaMeth.getParentClass();
			HashMap<String, String> genericAndErasureTypeParameters = QDoxUtil.getGenericAndErasureTypeParametersForMethod(onwerType, javaMeth);
			Set<String> genericTypes = genericAndErasureTypeParameters.keySet();

			for (Iterator<String> iterator2 = genericTypes.iterator(); iterator2
					.hasNext();) {
				String genericType = iterator2.next();
				String erasureType = genericAndErasureTypeParameters.get(genericType);
				currentMethErasureTmp = currentMethErasureTmp.replaceAll("(\\b)"+genericType+"(\\b)", erasureType);
				currentMethErasure = currentMethErasureTmp;
			}
			
			indexMeth++;
			if(!currentMethErasure.equals("")){
				int openBraceIndex = currentMethErasure.indexOf('{');
				if(openBraceIndex != -1){					
					// means that we have generic type parameters 
					if(currentMethErasure.contains("<") && currentMethErasure.contains(">")){
						int lp = currentMethErasure.lastIndexOf('(');
						String currentMethErasurePreArgs = currentMethErasure.substring(0, lp);
						String currentMethErasureArgs = currentMethErasure.substring(lp);
						String currentMethPreArgsWithoutDiamond = currentMethErasurePreArgs.replaceAll("\\<[\\w\\W]*\\>", "");
						String currentMethArgsWithoutDiamond = currentMethErasureArgs;
						
						while(currentMethArgsWithoutDiamond.contains("<")){
							int lD = currentMethArgsWithoutDiamond.indexOf("<");
							int rD = currentMethArgsWithoutDiamond.indexOf(">");
							String diamond = currentMethArgsWithoutDiamond.substring(lD, (rD+1));
							currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(diamond, "");
						}
						
						if(currentMethArgsWithoutDiamond.contains(">")){
							currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(">", ""); // removing any residual from generics --- [[[hemr]]]
						}
						
						currentMethErasure = currentMethErasure.replace(currentMethErasurePreArgs, currentMethPreArgsWithoutDiamond).
								replace(currentMethErasureArgs, currentMethArgsWithoutDiamond);
					}
				}
				fileAsString = StringUtils.replaceOnce(fileAsString, currentMeth, currentMethErasure);
				// handling crosscutting specs
				JavaClass ownerType = javaMeth.getParentClass();
				QDoxUtil.handlingCrosscutSpecs(file, javaMeth, currentMethErasure, ownerType);
			}
			else{
				int openBraceIndex = currentMeth.indexOf('{');
				if(openBraceIndex != -1){
					String currentMethTmp = currentMeth;
					int lp = currentMeth.lastIndexOf('(');
					String currentMethPreArgs = currentMethTmp.substring(0, lp);
					String currentMethArgs = currentMethTmp.substring(lp);
					String currentMethPreArgsWithoutDiamond = currentMethPreArgs.replaceAll("\\<[\\w\\W]*\\>", "");
					String currentMethArgsWithoutDiamond = currentMethArgs;
					
					while(currentMethArgsWithoutDiamond.contains("<")){
						int lD = currentMethArgsWithoutDiamond.indexOf("<");
						int rD = currentMethArgsWithoutDiamond.indexOf(">");
						String diamond = currentMethArgsWithoutDiamond.substring(lD, (rD+1));
						currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(diamond, "");
					}
					
					if(currentMethArgsWithoutDiamond.contains(">")){
						currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(">", ""); // removing any residual from generics --- [[[hemr]]]
					}
					
					currentMethTmp = currentMethTmp.replace(currentMethPreArgs, currentMethPreArgsWithoutDiamond).
							replace(currentMethArgs, currentMethArgsWithoutDiamond);
					fileAsString = StringUtils.replaceOnce(fileAsString, currentMeth, currentMethTmp);
					// handling crosscutting specs
					JavaClass ownerType = javaMeth.getParentClass();
					QDoxUtil.handlingCrosscutSpecs(file, javaMeth, currentMethTmp, ownerType);
				}
				else{
					// means that we have generic type parameters 
					if(currentMeth.contains("<") && currentMeth.contains(">")){
						String currentMethTmp = currentMeth;
						int lp = currentMeth.lastIndexOf('(');
						String currentMethPreArgs = currentMethTmp.substring(0, lp);
						String currentMethArgs = currentMethTmp.substring(lp);
						String currentMethPreArgsWithoutDiamond = currentMethPreArgs.replaceAll("\\<[\\w\\W]*\\>", "");
						String currentMethArgsWithoutDiamond = currentMethArgs;
						
						while(currentMethArgsWithoutDiamond.contains("<")){
							int lD = currentMethArgsWithoutDiamond.indexOf("<");
							int rD = currentMethArgsWithoutDiamond.indexOf(">");
							String diamond = currentMethArgsWithoutDiamond.substring(lD, (rD+1));
							currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(diamond, "");
						}
						
						if(currentMethArgsWithoutDiamond.contains(">")){
							currentMethArgsWithoutDiamond = currentMethArgsWithoutDiamond.replace(">", ""); // removing any residual from generics --- [[[hemr]]]
						}
						
						currentMethTmp = currentMethTmp.replace(currentMethPreArgs, currentMethPreArgsWithoutDiamond).
								replace(currentMethArgs, currentMethArgsWithoutDiamond);
						fileAsString = StringUtils.replaceOnce(fileAsString, currentMeth, currentMethTmp);
						// handling crosscutting specs
						JavaClass ownerType = javaMeth.getParentClass();
						QDoxUtil.handlingCrosscutSpecs(file, javaMeth, currentMethTmp, ownerType);
					}
					else {
						// means that we have a method in a .jml file
						// handling crosscutting specs
						JavaClass ownerType = javaMeth.getParentClass();
						QDoxUtil.handlingCrosscutSpecs(file, javaMeth, currentMeth, ownerType);
					}
				}
			}
		}
		return fileAsString;
	}

	private static void handlingCrosscutSpecs(File file, JavaMethod javaMeth,
			String currentMeth, JavaClass ownerType) throws PositionedError {
		boolean isAspect = false;
//		boolean isCrosscutSpec = false;
//		Annotation xciAnno = null;
		for (int i = 0; i < ownerType.getAnnotations().length; i++) {
			if(ownerType.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.Aspect")){
				isAspect = true;
			}
//			if(ownerType.getAnnotations()[i].toString().startsWith("@org.jmlspecs.annotation.AspectXCS")){
//				isCrosscutSpec = true;
//				xciAnno = ownerType.getAnnotations()[i];
//			}
		}
//		if(isAspect && isCrosscutSpec){
//			System.out.println("parsedMethTmp = "+currentMeth);
//			if(!ownerType.isInner()){
//				TokenReference tref = TokenReference.build(file, xciAnno.getLineNumber());
//				throw new PositionedError(tref, JmlMessages.INVALID_XCS_ANNOTATION_LOCATION, "");
//			}
//			// is to handle crosscut specs?
//			if(Main.aRacOptions.crosscuttingContractSpecifications()){
//				JavaDocBuilder qDoxJavaMethodParserForCrosscutSpecs = new JavaDocBuilder();
//				qDoxJavaMethodParserForCrosscutSpecs.addSource(new StringReader("public class JMethodParser{\n"+currentMeth+"\n}"));
//				JavaClass parsedClassTmp = qDoxJavaMethodParserForCrosscutSpecs.getClassByName("JMethodParser");
//				if(parsedClassTmp.getMethods().length == 1){
//					JavaMethod parsedMethTmp = parsedClassTmp.getMethods()[0];
//					parsedMethTmp.setAnnotations(javaMeth.getAnnotations());
//					parsedMethTmp.setParentClass(ownerType);
//					boolean isToInclude = false;
//					for (int i = 0; i < parsedMethTmp.getAnnotations().length; i++) {
//						if(parsedMethTmp.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation")){
//							isToInclude = true;
//						}
//					}
//					if(isToInclude){
//						AspectUtil.getInstance().appendCrosscutSpecMeth(parsedMethTmp);
//					}
//				}
//			}
//		}
		 if(isAspect){
			// means that we can have advice and pointcuts but is not a part of the crosscutting contract specification
			if(Main.aRacOptions != null){
				if(Main.aRacOptions.adviceChecking()){
					AspectUtil.getInstance().appendAspectDecl(ownerType);
					JavaDocBuilder qDoxJavaMethodParserForCrosscutSpecs = new JavaDocBuilder();
					qDoxJavaMethodParserForCrosscutSpecs.addSource(new StringReader("public class JMethodParser{\n"+currentMeth+"\n}"));
					// adding Java imports
					for (int i = 0; i < ownerType.getSource().getImports().length; i++) {
						qDoxJavaMethodParserForCrosscutSpecs.getSources()[0].addImport(ownerType.getSource().getImports()[i]);
					}
					JavaClass parsedClassTmp = qDoxJavaMethodParserForCrosscutSpecs.getClassByName("JMethodParser");
					if(parsedClassTmp.getMethods().length == 1){
						JavaMethod parsedMethTmp = parsedClassTmp.getMethods()[0];
						parsedMethTmp.setAnnotations(javaMeth.getAnnotations());
						parsedMethTmp.setParentClass(ownerType);
						// handling advice specs --- [[[hemr]
						boolean isToInclude = false;
						for (int i = 0; i < parsedMethTmp.getAnnotations().length; i++) {
							if(parsedMethTmp.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.A")){
								isToInclude = true;
							}
							if(parsedMethTmp.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.Before")){
								isToInclude = true;
							}
						}
						if(isToInclude){
							AspectUtil.getInstance().appendAspectAdvice(parsedMethTmp);
						}
						// handling crosscutting contract specs for advice --- [[[hemr]
						isToInclude = false;
						boolean isPC = false;
						for (int i = 0; i < parsedMethTmp.getAnnotations().length; i++) {
							if(parsedMethTmp.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.Pointcut")){
								isPC = true;
							}
						}
						isToInclude = isPC;
						if(isToInclude){
							AspectUtil.getInstance().appendCrosscutSpecMeth(parsedMethTmp);
						}
					}
				}
			}
		}
		else {
			// is to handle crosscut specs?
			if(Main.aRacOptions != null){
				if(Main.aRacOptions.crosscuttingContractSpecifications()){
					JavaDocBuilder qDoxJavaMethodParserForCrosscutSpecs = new JavaDocBuilder();
					qDoxJavaMethodParserForCrosscutSpecs.addSource(new StringReader("public class JMethodParser{\n"+currentMeth+"\n}"));
					// adding Java imports
					for (int i = 0; i < ownerType.getSource().getImports().length; i++) {
						qDoxJavaMethodParserForCrosscutSpecs.getSources()[0].addImport(ownerType.getSource().getImports()[i]);
					}
					JavaClass parsedClassTmp = qDoxJavaMethodParserForCrosscutSpecs.getClassByName("JMethodParser");
					if(parsedClassTmp.getMethods().length == 1){
						JavaMethod parsedMethTmp = parsedClassTmp.getMethods()[0];
						parsedMethTmp.setAnnotations(javaMeth.getAnnotations());
						parsedMethTmp.setParentClass(ownerType);
						boolean isToInclude = false;
						boolean isPC = false;
						for (int i = 0; i < parsedMethTmp.getAnnotations().length; i++) {
							if(parsedMethTmp.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.Pointcut")){
								isPC = true;
							}
						}
						isToInclude = isPC;
						if(isToInclude){
							AspectUtil.getInstance().appendCrosscutSpecMeth(parsedMethTmp);
						}
					}
				}
			}
		}
	}
	
	public static boolean iPCXCS(JavaMethod javaMeth){
		boolean isPCXCS = false;
		boolean isPC = false;
		for (int i = 0; i < javaMeth.getAnnotations().length; i++) {
			if((javaMeth.getAnnotations()[i].toString().startsWith("@org.aspectj.lang.annotation.Pointcut"))){
				isPC = true;
			}
		}
		isPCXCS = isPC;
		return isPCXCS;
	}

	private static String getFileEnumTypeErasureProcessingAsString(
			StringBuffer bufferedFile, String fileAsString,
			List<JavaClass> javaDeclEnumTypes) throws IOException {
		BufferedReader buffer = null;
		String line = "";
		fileAsString = fileAsString.replace("enum ", "final class ");

		//		handling enumeration fields
		for (Iterator<JavaClass> iterator = javaDeclEnumTypes.iterator(); iterator
				.hasNext();) {
			StringBuffer enumDeclArea = new StringBuffer("");
			JavaClass enumType = iterator.next();
			List<JavaField> enumTypeFields = QDoxUtil.getADeclaredJavaEnumerationFieldsInEnumType(enumType);
			if(enumTypeFields.size() > 0){
				String firstEnumField = enumTypeFields.get(0).getName();
				String enumFieldsReplacement = QDoxUtil.getADeclaredJavaEnumerationFieldsInEnumTypeAsErasure(enumType);
				int targetLineNumber = enumType.getLineNumber();
				buffer = new BufferedReader(new StringReader(fileAsString));
				bufferedFile.delete(0, (bufferedFile.length()-1)); // resetting buffer
				line = buffer.readLine();
				int fileLineNumber = 1;
				int appendedLineNumbers = 0;
				boolean firstFieldDetected = false;
				boolean finished = false;
				while (line != null) {
					if((fileLineNumber >= targetLineNumber) && (!finished)){ 
						if(!firstFieldDetected){
							if(line.contains(firstEnumField)){
								firstFieldDetected = true;
								enumDeclArea.append(line.substring(line.indexOf(firstEnumField)));
								enumDeclArea.append("\n");
								if ((enumDeclArea.toString().contains(";"))
										|| (enumDeclArea.toString()
												.contains("}"))) {
									finished = true;
								}
								appendedLineNumbers++;
							}
						}
						else {
							enumDeclArea.append(line);
							enumDeclArea.append("\n");
							if ((enumDeclArea.toString().contains(";"))
									|| (enumDeclArea.toString().contains("}"))) {
								finished = true;
							}
							appendedLineNumbers++;
						}
					}
					bufferedFile.append(line);
					bufferedFile.append("\n");
					line = buffer.readLine();
					fileLineNumber++;
				}
				
				// updating the current enum type area in the file
				String enumDeclAreaStr = "";
				if(enumDeclArea.toString().contains("}")){
					enumDeclAreaStr = enumDeclArea.toString().substring(0, enumDeclArea.toString().lastIndexOf("}"));
				}
				else{
					enumDeclAreaStr = enumDeclArea.toString();
				}
				fileAsString = StringUtils.replaceOnce(fileAsString, enumDeclAreaStr, "/* "+enumDeclAreaStr+" */"+enumFieldsReplacement);
			}
		}

		return fileAsString;
	}

	private static List<String> getAllJavaMethodDeclLexicallyInFile(
			StringBuffer bufferedFile, String fileAsString,
			List<JavaMethod> javaDeclMeths, List<String> javaDeclMethsAnnotationArea) throws IOException {
		List<String> fileMeths = new ArrayList<String>();
		BufferedReader buffer;
		String line;
		for (Iterator<JavaMethod> iterator = javaDeclMeths.iterator(); iterator
				.hasNext();) {
			JavaMethod javaMethod = iterator.next();
			int lineNumber = javaMethod.getLineNumber();
			buffer = new BufferedReader(new StringReader(fileAsString));
			line = buffer.readLine();
			int fileLineNumber = 1;
			boolean canStart = false;
			while (line != null) {
				if(lineNumber == fileLineNumber){ canStart = true;}
				if(canStart){
					bufferedFile.append(line);
					bufferedFile.append("\n");
					JavaDocBuilder qDoxJavaMethodParser = new JavaDocBuilder();
					try {
						qDoxJavaMethodParser.addSource(new StringReader("public class JMethodParser{\n"+bufferedFile.toString()+"\n}"));
						if(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods().length == 1){
							String methDecl = bufferedFile.toString();
							methDecl = QDoxUtil.getMethDeclWithoutAnnotations(methDecl, javaDeclMethsAnnotationArea).trim();
							fileMeths.add(methDecl);
							bufferedFile.delete(0, (bufferedFile.length()-1));
							break;
						}
					} catch (Exception e) {
						// continue if it is not a valid method... [[[hemr]]]
					}
				}
				line = buffer.readLine();
				fileLineNumber++;
			}
		}
		return fileMeths;
	}
	
	private static List<String> getAllJavaMethodDeclLexicallyInFile(
			StringBuffer bufferedFile, String fileAsString,
			List<JavaMethod> javaDeclMeths) throws IOException {
		List<String> fileMeths = new ArrayList<String>();

		int count = 0;
		int size = (javaDeclMeths.size() - 1);
		for (Iterator<JavaMethod> iterator = javaDeclMeths.iterator(); iterator
				.hasNext();) {
			JavaMethod javaMethod = iterator.next();
			int lineNumberStart = javaMethod.getLineNumber();
			int lineNumberEnd = -1;
			if(count < size){
				lineNumberEnd = javaDeclMeths.get(count + 1).getLineNumber();
			}
			String meth = "";
			while(meth.equals("") && lineNumberStart >= 1){
				String fileSnippet = getFileSnippet(fileAsString, lineNumberStart, lineNumberEnd);
				meth = getMethodLexicallyInFileFromSnippet(fileSnippet, javaMethod);
				if(meth.equals("")){
					lineNumberStart = lineNumberStart - 1;
				}
				else{
					fileMeths.add(meth);
				}
			}
			count ++;
		}
		return fileMeths;
	}
	
	private static String getFileSnippet(String fileAsString,
			int lineNumberStart, int lineNumberEnd) throws IOException {
		BufferedReader buffer;
		String line;
		StringBuffer bufferedFile = new StringBuffer("");
		buffer = new BufferedReader(new StringReader(fileAsString));
		line = buffer.readLine();
		int fileLineNumber = 1;
		boolean canStart = false;
		if(lineNumberEnd != -1){
			while (line != null) {
				if(lineNumberStart == fileLineNumber){ canStart = true;}
				if(canStart && (lineNumberEnd >= fileLineNumber) ){
					bufferedFile.append(line);
					bufferedFile.append("\n");
				}
				line = buffer.readLine();
				fileLineNumber++;
			}
			buffer.close();
		}
		else {
			while (line != null) {
				if(lineNumberStart == fileLineNumber){ canStart = true;}
				if(canStart){
					bufferedFile.append(line);
					bufferedFile.append("\n");
				}
				line = buffer.readLine();
				fileLineNumber++;
			}
			buffer.close();
		}
		
		return bufferedFile.toString();
	}
	
	private static String getMethodLexicallyInFileFromSnippet(String fileSnippet, JavaMethod javaMethod) throws IOException {
		String currentMeth = "";
		BufferedReader buffer;
		String line;
		StringBuffer bufferedFile = new StringBuffer("");
		StringBuffer bufferedFileReverse = new StringBuffer("");
		buffer = new BufferedReader(new StringReader(fileSnippet));
		line = buffer.readLine();
		while (line != null) {
			bufferedFile.append(line);
			bufferedFile.append("\n");
			bufferedFileReverse.insert(0, line);
			bufferedFileReverse.insert(0, "\n");
			JavaDocBuilder qDoxJavaMethodParser = new JavaDocBuilder();
			try {
				//				bufferedFile.toString().replaceAll("(/\\*(.|[\\r\\n])*?\\*/)"
				qDoxJavaMethodParser.addSource(new StringReader("public class JMethodParser{\n"+bufferedFile.toString()+"\n}"));
				if(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods().length == 1){
					if(javaMethod.getName().equals(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods()[0].getName())){
						//						System.out.println("natural match = "+bufferedFile.toString());
						if(javaMethod.getAnnotations().length > 0){
							boolean match = false;
							buffer = new BufferedReader(new StringReader(bufferedFileReverse.toString()));
							StringBuffer sb = new StringBuffer("");
							line = buffer.readLine();
							while (line != null) {
								sb.insert(0, line);
								sb.insert(0, "\n");
								qDoxJavaMethodParser = new JavaDocBuilder();
								try {
									qDoxJavaMethodParser.addSource(new StringReader("public class JMethodParser{\n"+sb.toString()+"\n}"));
									if(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods().length == 1){
										if(javaMethod.getName().equals(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods()[0].getName())){
											//											System.out.println("reverse match = "+sb.toString());
											currentMeth = sb.toString();
											match = true;
											break;
										}
									}
								} catch (Exception e) {
									// continue if it is not a valid method... [[[hemr]]]
								}
								line = buffer.readLine();
							}
							if(match){
								break;
							}
						}
						else{
							currentMeth = bufferedFile.toString();
							break;
						}
					}
				}	
				//					
			} catch (Exception e) {
				// continue if it is not a valid method... [[[hemr]]]
			}
			line = buffer.readLine();
		}
		return currentMeth;
	}

//	private static String getMethodLexicallyInFile(StringBuffer bufferedFile,
//			String fileAsString, JavaMethod javaMethod,
//			int lineNumber) throws IOException {
//		String currentMeth = "";
//		BufferedReader buffer;
//		String line;
//		buffer = new BufferedReader(new StringReader(fileAsString));
//		line = buffer.readLine();
//		int fileLineNumber = 1;
//		boolean canStart = false;
//		Stack methDeclInv = new Stack();
//		while (line != null) {
//			if(lineNumber == fileLineNumber){ canStart = true;}
//			if(canStart){
//				bufferedFile.append(line);
//				bufferedFile.append("\n");
//				methDeclInv.push(line);
//				methDeclInv.push("\n");
//				JavaDocBuilder qDoxJavaMethodParser = new JavaDocBuilder();
//				try {
//					qDoxJavaMethodParser.addSource(new StringReader("public class JMethodParser{\n"+bufferedFile.toString().replace(";", "")+"\n}"));
//					if(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods().length == 1){
//						String methDecl = bufferedFile.toString();
//						bufferedFile.delete(0, (bufferedFile.length()-1));
//						if(javaMethod.getAnnotations().length > 0){
//							StringBuffer sb = new StringBuffer("");
//							while(methDeclInv.size() > 0){
//								sb.insert(0, (String)methDeclInv.pop());
//								try {
//									qDoxJavaMethodParser.addSource(new StringReader("public class JMethodParser{\n"+sb.toString().replace(";", "")+"\n}"));
//									if(qDoxJavaMethodParser.getClassByName("JMethodParser").getMethods().length == 1){
//										methDecl = sb.toString();
//										currentMeth = methDecl;
//										break;
//									}
//								} catch (Exception e) {
//									// continue if it is not a valid method... [[[hemr]]]
//								}
//							}
//							break;
//						}
//						else{
//							currentMeth = methDecl;
//							break;
//						}
//					}
//				} catch (Exception e) {
//					// continue if it is not a valid method... [[[hemr]]]
//				}
//			}
//			line = buffer.readLine();
//			fileLineNumber++;
//		}
//		return currentMeth;
//	}

	private static String getMethodAnnotationAreaCommentedProcessedWithJML(
			JavaMethod annotatedJavaMethod, StringBuffer annotationAreaCommented) {
		String annotationAreaCommentedStr = annotationAreaCommented.toString();
		Annotation[] methAnnotation = annotatedJavaMethod.getAnnotations();

		for (int i = 0; i < methAnnotation.length; i++) {
			Annotation currentAnnotation = methAnnotation[i];
			// handling single JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Requires")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "requires_redundantly" : "requires") : "requires");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Assignable")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "assignable_redundantly" : "assignable") : "assignable");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Ensures")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "ensures_redundantly" : "ensures") : "ensures");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Signals")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "signals_redundantly" : "signals") : "signals");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SignalsOnly")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "signals_only_redundantly" : "signals_only") : "signals_only");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Callable")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "callable_redundantly" : "callable") : "callable");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Accessible")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "accessible_redundantly" : "accessible") : "accessible");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Duration")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "duration_redundantly" : "duration") : "duration");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Captures")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "captures_redundantly" : "captures") : "captures");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.WorkingSpace")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					String clauseKind = (currentAnnotation.getNamedParameterMap().containsKey("redundantly")? 
							(currentAnnotation.getProperty("redundantly").toString().equals("true")? "working_space_redundantly" : "working_space") : "working_space");
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					String pred = QDoxUtil.preparePred(currentAnnotation.getProperty("value").toString());
					String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
			}

			// handling composite JML annotations
			if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.RequiresDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> requiresDefinitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = requiresDefinitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "requires_redundantly" : "requires") : "requires");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.AssignableDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "assignable_redundantly" : "assignable") : "assignable");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.EnsuresDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "ensures_redundantly" : "ensures") : "ensures");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SignalsDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "signals_redundantly" : "signals") : "signals");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SignalsOnlyDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "signals_only_redundantly" : "signals_only") : "signals_only");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.CallableDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "callable_redundantly" : "callable") : "callable");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.AccessibleDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "accessible_redundantly" : "accessible") : "accessible");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.DurationDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "duration_redundantly" : "duration") : "duration");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.CapturesDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "captures_redundantly" : "captures") : "captures");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.WorkingSpaceDefinitions")){
				@SuppressWarnings("unchecked")
				ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
				for (Iterator<Annotation> iterator2 = definitions
						.iterator(); iterator2.hasNext();) {
					Annotation currentDef =  iterator2.next();
					if(currentDef.getNamedParameterMap().containsKey("value")){
						String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
								(currentDef.getProperty("redundantly").toString().equals("true")? "working_space_redundantly" : "working_space") : "working_space");
						String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
						String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
						String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
						annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
					}
				}
			}
			// speccase
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SpecCase")){
				if(currentAnnotation.getNamedParameterMap().containsKey("header")){
					String header = QDoxUtil.preparePred((String)currentAnnotation.getProperty("header").getParameterValue());
					String clause = "/*@ "+header+" @*/";
					String preLineNumber = "/*"+currentAnnotation.getLineNumber()+"*/";
					annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("requiresDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("requiresDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "requires_redundantly" : "requires") : "requires");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("assignableDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("assignableDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "assignable_redundantly" : "assignable") : "assignable");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("ensuresDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("ensuresDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "ensures_redundantly" : "ensures") : "ensures");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("signalsDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("signalsDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "signals_redundantly" : "signals") : "signals");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("signalsOnlyDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("signalsOnlyDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "signals_only_redundantly" : "signals_only") : "signals_only");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("callableDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("callableDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "callable_redundantly" : "callable") : "callable");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("accessibleDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("accessibleDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "accessible_redundantly" : "accessible") : "accessible");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("durationDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("durationDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "duration_redundantly" : "duration") : "duration");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("capturesDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("capturesDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "captures_redundantly" : "captures") : "captures");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
				if(currentAnnotation.getNamedParameterMap().containsKey("workingSpaceDefinitions")){
					@SuppressWarnings("unchecked")
					ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("workingSpaceDefinitions").getParameterValue();
					for (Iterator<Annotation> iterator2 = definitions
							.iterator(); iterator2.hasNext();) {
						Annotation currentDef =  iterator2.next();
						if(currentDef.getNamedParameterMap().containsKey("value")){
							String clauseKind = (currentDef.getNamedParameterMap().containsKey("redundantly")? 
									(currentDef.getProperty("redundantly").toString().equals("true")? "working_space_redundantly" : "working_space") : "working_space");
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String pred = QDoxUtil.preparePred(currentDef.getProperty("value").toString());
							String clause = "/*@ "+clauseKind+" "+pred+"; @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
						}
					}
				}
			}//
			// also
			else if(currentAnnotation.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.Also")){
				if(currentAnnotation.getNamedParameterMap().containsKey("value")){
					if(currentAnnotation.getProperty("value").getParameterValue() instanceof ArrayList){
						@SuppressWarnings("unchecked")
						ArrayList<Annotation> definitions = (ArrayList<Annotation>)currentAnnotation.getProperty("value").getParameterValue();
						for (Iterator<Annotation> iterator2 = definitions
								.iterator(); iterator2.hasNext();) {
							Annotation currentDef =  iterator2.next();
							if(currentDef.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SpecCase")){
								String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
								String clause = "/*@ also @*/";
								annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
								
								if(currentDef.getNamedParameterMap().containsKey("header")){
									String header = QDoxUtil.preparePred((String)currentDef.getProperty("header").getParameterValue());
									clause = "/*@ "+header+" @*/";
									preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
									annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
								}

								if(currentDef.getNamedParameterMap().containsKey("requiresDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("requiresDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "requires_redundantly" : "requires") : "requires");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("assignableDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("assignableDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "assignable_redundantly" : "assignable") : "assignable");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("ensuresDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("ensuresDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "ensures_redundantly" : "ensures") : "ensures");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("signalsDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("signalsDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "signals_redundantly" : "signals") : "signals");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("signalsOnlyDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("signalsOnlyDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "signals_only_redundantly" : "signals_only") : "signals_only");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("callableDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("callableDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "callable_redundantly" : "callable") : "callable");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("accessibleDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("accessibleDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "accessible_redundantly" : "accessible") : "accessible");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("durationDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("durationDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "duration_redundantly" : "duration") : "duration");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("capturesDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("capturesDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "captures_redundantly" : "captures") : "captures");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}

								if(currentDef.getNamedParameterMap().containsKey("workingSpaceDefinitions")){
									@SuppressWarnings("unchecked")
									ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("workingSpaceDefinitions").getParameterValue();
									for (Iterator<Annotation> iterator3 = definitions2
											.iterator(); iterator3.hasNext();) {
										Annotation currentDef2 =  iterator3.next();
										if(currentDef2.getNamedParameterMap().containsKey("value")){
											String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
													(currentDef2.getProperty("redundantly").toString().equals("true")? "working_space_redundantly" : "working_space") : "working_space");
											String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
											String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
											String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
											annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
										}
									}
								}
							}
						}
					}
					else{
						Annotation currentDef = (Annotation)currentAnnotation.getProperty("value").getParameterValue();
						if(currentDef.getType().getFullyQualifiedName().equals("org.jmlspecs.annotation.SpecCase")){
							String preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
							String clause = "/*@ also @*/";
							annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
							
							if(currentDef.getNamedParameterMap().containsKey("header")){
								String header = QDoxUtil.preparePred((String)currentDef.getProperty("header").getParameterValue());
								clause = "/*@ "+header+" @*/";
								preLineNumber = "/*"+currentDef.getLineNumber()+"*/";
								annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber, clause+preLineNumber);
							}

							if(currentDef.getNamedParameterMap().containsKey("requiresDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("requiresDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "requires_redundantly" : "requires") : "requires");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("assignableDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("assignableDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "assignable_redundantly" : "assignable") : "assignable");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("ensuresDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("ensuresDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "ensures_redundantly" : "ensures") : "ensures");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("signalsDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("signalsDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "signals_redundantly" : "signals") : "signals");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("signalsOnlyDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("signalsOnlyDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "signals_only_redundantly" : "signals_only") : "signals_only");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("callableDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("callableDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "callable_redundantly" : "callable") : "callable");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("accessibleDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("accessibleDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "accessible_redundantly" : "accessible") : "accessible");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("durationDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("durationDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "duration_redundantly" : "duration") : "duration");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("capturesDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("capturesDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "captures_redundantly" : "captures") : "captures");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}

							if(currentDef.getNamedParameterMap().containsKey("workingSpaceDefinitions")){
								@SuppressWarnings("unchecked")
								ArrayList<Annotation> definitions2 = (ArrayList<Annotation>)currentDef.getProperty("workingSpaceDefinitions").getParameterValue();
								for (Iterator<Annotation> iterator3 = definitions2
										.iterator(); iterator3.hasNext();) {
									Annotation currentDef2 =  iterator3.next();
									if(currentDef2.getNamedParameterMap().containsKey("value")){
										String clauseKind = (currentDef2.getNamedParameterMap().containsKey("redundantly")? 
												(currentDef2.getProperty("redundantly").toString().equals("true")? "working_space_redundantly" : "working_space") : "working_space");
										String preLineNumber2 = "/*"+currentDef2.getLineNumber()+"*/";
										String pred = QDoxUtil.preparePred(currentDef2.getProperty("value").toString());
										String clause2 = "/*@ "+clauseKind+" "+pred+"; @*/";
										annotationAreaCommentedStr = annotationAreaCommentedStr.replace(preLineNumber2, clause2+preLineNumber2);
									}
								}
							}
						}
					}
				}
			}
		}
		return annotationAreaCommentedStr;
	}
	
	private static String getMethDeclWithoutAnnotations(String methDecl, List<String> javaDeclMethsAnnotationArea){
		for (Iterator<String> iterator = javaDeclMethsAnnotationArea.iterator(); iterator
				.hasNext();) {
			String header = iterator.next();
			methDecl = methDecl.replace(header, "");
		}
		return methDecl;
	}
	
	private static boolean isJMLAnnotation(String annotation){
		boolean result = false;
		if(annotation.equals("@Model()") ||
			   annotation.equals("@Nullable()") ||
				annotation.equals("@NonNull()")||
				annotation.equals("@SpecPublic()")||
				annotation.equals("@SpecProtected()")||
				annotation.equals("@Pure()")){
			result = true;
		}
		return result;
	}
	
	private static boolean isParenBalanced(String token){
		boolean ret = false;
		Pattern lParenPattern = Pattern.compile("\\(");
		Matcher lParenMatcher = lParenPattern.matcher(token);
		int lParenLineCount = 0;
		while(lParenMatcher.find()){
			lParenLineCount++;
		}
		Pattern rParenPattern = Pattern.compile("\\)");
		Matcher rParenMatcher = rParenPattern.matcher(token);
		int rParenLineCount = 0;
		while(rParenMatcher.find()){
			rParenLineCount++;
		}

		if(lParenLineCount == rParenLineCount){
			ret = true;
		}
		return ret;
	}
	
	private static String removeNonJMLAnnotationParameters(String currentMethTmp) {
		String currentMethTmp2 = currentMethTmp;
		
		Pattern tmpPattern = Pattern.compile("/\\*(.|\\s)*@(.|\\s)*\\*/");
		Matcher tmpMatcher = tmpPattern.matcher(currentMethTmp2);
		while(tmpMatcher.find()){
			currentMethTmp2 = currentMethTmp2.replace(tmpMatcher.group(), "");
		}
		
		Pattern annoPattern = Pattern.compile("@(\\s)*(.)*(\\s)*(\\((.)*\\))");
		Matcher annoMatcher = annoPattern.matcher(currentMethTmp2);
		while(annoMatcher.find()){
			String group = annoMatcher.group();
			char [] groupAsChar = group.toCharArray();
			boolean canCheck = false;
			boolean foundAt = false;
			StringBuffer matchedAnnotation = new StringBuffer("");
			for (int i = 0; i < groupAsChar.length; i++) {
				if(groupAsChar[i] == '('){
					canCheck = true;
				}
				if(groupAsChar[i] == '@'){
					foundAt = true;
				}
				if(foundAt){
					matchedAnnotation.append(groupAsChar[i]);	
				}
				if(canCheck){
					if(QDoxUtil.isParenBalanced(matchedAnnotation.toString())){
						// if is not JML annotation, we remove it...
						if(!isJMLAnnotation(matchedAnnotation.toString())){
							currentMethTmp = StringUtils.replaceOnce(currentMethTmp, matchedAnnotation.toString(), "");	
						}
						matchedAnnotation = new StringBuffer("");
						canCheck = false;
						foundAt = false;
					}
				}
			}
		}
		return currentMethTmp;
	}
	
	private static int getLineNumbersQtd(String tmp) {
		int newLineCount = 0;
		
		BufferedReader buffer = new BufferedReader(new StringReader(tmp));
		String line = null;
		try {
			line = buffer.readLine();
			while (line != null) {
				line = buffer.readLine();
				newLineCount++;
			}
			buffer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return newLineCount-1;
	}

//	private static String getNewLinesCaracter(int numberOfNewLines, boolean isPC){
//		StringBuffer result = null;
//		if(isPC){
//			result = new StringBuffer("/* #PC */");
//		}
//		else{
//			result = new StringBuffer("");
//		}
//		for (int i = 0; i < numberOfNewLines; i++) {
//			result.append("\n");
//		}
//		return result.toString();
//	}
	
	private static String getNewLinesCaracter(int numberOfNewLines){
		StringBuffer result = null;
		result = new StringBuffer("");
		for (int i = 0; i < numberOfNewLines; i++) {
			result.append("\n");
		}
		return result.toString();
	}

	private static String preparePred(String clause){
		Pattern pattern = Pattern.compile("(\"[^\"]*\")*");
		Matcher patterhMatcher = pattern.matcher(clause);
		StringBuffer clauseParsed = new StringBuffer("");
		while(patterhMatcher.find()){
			clauseParsed.append(patterhMatcher.group());
		}
		return clauseParsed.toString().replace("\"", "").replace("#dq", "\"").replace("#sq", "\'").replace("#", "\\");
	}

	private static String defaultValue(String type) {
		String result = "";

		if(!type.equals("void")){
			if(type.equals("boolean")){
				result = "false";
			}
			else if(type.equals("byte")){
				result = "(byte) 0";
			}
			else if(type.equals("short")){
				result = "(short) 0";
			}
			else if(type.equals("int")){
				result = "0";
			}
			else if(type.equals("long")){
				result = "0L";
			}
			else if(type.equals("float")){
				result = "0.0f";
			}
			else if(type.equals("double")){
				result = "0.0d";
			}
			else if(type.equals("char")){
				result = "(char) 0";
			}
			else{
				result = "null";
			}
		}
		return result;
	}
	
//	private static boolean isWordPartOfMethDecl(String lineWord, JavaClass annotatedJavaType){
//		boolean result = false;
//		if(annotatedJavaType.isPublic()){
//			if(lineWord.equals("public")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isProtected()){
//			if(lineWord.equals("protected")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isPrivate()){
//			if(lineWord.equals("private")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isStatic()){
//			if(lineWord.equals("static")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isAbstract()){
//			if(lineWord.equals("abstract")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isFinal()){
//			if(lineWord.equals("final")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isEnum()){
//			if(lineWord.equals("enum")){
//				result = true;
//			}
//		}
//		if(annotatedJavaType.isInterface()){
//			if(lineWord.equals("interface")){
//				result = true;
//			}
//		}
//		// In case of @annotation type
//		if(lineWord.equals("@interface")){
//			result = true;
//		}
//		if(!annotatedJavaType.isEnum() && 
//				!annotatedJavaType.isInterface()){
//			if(lineWord.equals("class")){
//				result = true;
//			}
//		}
//		return result;
//	}
//	
//	private static boolean isWordPartOfFieldDecl(String lineWord, JavaField annotatedJavaField){
//		boolean result = false;
//		if(annotatedJavaField.isPublic()){
//			if(lineWord.equals("public")){
//				result = true;
//			}
//		}
//		if(annotatedJavaField.isProtected()){
//			if(lineWord.equals("protected")){
//				result = true;
//			}
//		}
//		if(annotatedJavaField.isPrivate()){
//			if(lineWord.equals("private")){
//				result = true;
//			}
//		}
//		if(annotatedJavaField.isStatic()){
//			if(lineWord.equals("static")){
//				result = true;
//			}
//		}
//		if(annotatedJavaField.isAbstract()){
//			if(lineWord.equals("abstract")){
//				result = true;
//			}
//		}
//		if(annotatedJavaField.isFinal()){
//			if(lineWord.equals("final")){
//				result = true;
//			}
//		}
//		// handling field type name
//		String shortTypeName = annotatedJavaField.getType().getFullyQualifiedName().
//				substring(annotatedJavaField.getType().getFullyQualifiedName().lastIndexOf(".")+1);
//		if(lineWord.equals(annotatedJavaField.getType().getFullyQualifiedName())){
//			result = true;
//		}
//		if(lineWord.equals(shortTypeName)){
//			result = true;
//		}
//		return result;
//	}
	
	// Util using JavaParser
	
	//	private static int getMemberLineStartWithJavaParser(int lineEnd, String fileAsString){
	//		int lineStart = 0;
	//		CompilationUnit cu = null;
	//		try {
	//			cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
	//		} catch (UnsupportedEncodingException | ParseException e) {
	//			e.printStackTrace();
	//		}
	//		for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
	//			TypeDeclaration type = (TypeDeclaration) iterator.next();
	//			for (Iterator iterator2 = type.getMembers().iterator(); iterator2
	//					.hasNext();) {
	//				BodyDeclaration member = (BodyDeclaration) iterator2.next();
	//				if(member instanceof FieldDeclaration){
	//					if(lineEnd == ((FieldDeclaration)member).getEndLine()){
	//						lineStart = ((FieldDeclaration)member).getBeginLine();
	//					}
	//				}
	//			}
	//		}
	//		return lineStart;
	//	}

	private static int getFieldAnnotationLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		
		outer:
			for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
				TypeDeclaration type = (TypeDeclaration) iterator.next();
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2
							.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof FieldDeclaration){
							if(lineStart == ((FieldDeclaration)member).getBeginLine()){
								lineEnd = ((FieldDeclaration)member).getAnnotations().get(((FieldDeclaration)member).getAnnotations().size() - 1).getEndLine();
								break;
							}
						}
						else if(member instanceof TypeDeclaration){
							lineEnd = QDoxUtil.getFieldAnnotationLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
							if(lineEnd != 0){
								break outer;
							}
						}
					}
			}

		return lineEnd;
	}

	private static int getFieldAnnotationLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(type.getMembers() != null)
			for (Iterator iterator2 = type.getMembers().iterator(); iterator2
					.hasNext();) {
				BodyDeclaration member = (BodyDeclaration) iterator2.next();
				if(member instanceof FieldDeclaration){
					if(lineStart == ((FieldDeclaration)member).getBeginLine()){
						lineEnd = ((FieldDeclaration)member).getAnnotations().get(((FieldDeclaration)member).getAnnotations().size() - 1).getEndLine();
						break;
					}
					else if(member instanceof TypeDeclaration){
						lineEnd = QDoxUtil.getFieldAnnotationLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
						if(lineEnd != 0){
							break;
						}
					}
				}
			}

		return lineEnd;
	}

	private static int getFieldLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		
		outer:
			for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
				TypeDeclaration type = (TypeDeclaration) iterator.next();
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2
							.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof FieldDeclaration){
							if(lineStart == ((FieldDeclaration)member).getBeginLine()){
								lineEnd = ((FieldDeclaration)member).getEndLine();
								break;
							}
						}
						else if(member instanceof TypeDeclaration){
							lineEnd = QDoxUtil.getFieldLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
							if(lineEnd != 0){
								break outer;
							}
						}
					}
			}

		return lineEnd;
	}

	private static int getFieldLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(type.getMembers() != null)
			for (Iterator iterator2 = type.getMembers().iterator(); iterator2
					.hasNext();) {
				BodyDeclaration member = (BodyDeclaration) iterator2.next();
				if(member instanceof FieldDeclaration){
					if(lineStart == ((FieldDeclaration)member).getBeginLine()){
						lineEnd = ((FieldDeclaration)member).getEndLine();
						break;
					}
				}
				else if(member instanceof TypeDeclaration){
					lineEnd = QDoxUtil.getFieldLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
					if(lineEnd != 0){
						break;
					}
				}
			}

		return lineEnd;
	}

	private static int getMethodAnnotationLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		
		outer:
			for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
				TypeDeclaration type = (TypeDeclaration) iterator.next();
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2
							.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof ConstructorDeclaration){
							if(lineStart == ((ConstructorDeclaration)member).getBeginLine()){
								lineEnd = ((ConstructorDeclaration)member).getAnnotations().get(((ConstructorDeclaration)member).getAnnotations().size() - 1).getEndLine();
								break;
							}
						}
						else if(member instanceof MethodDeclaration){
							if(lineStart == ((MethodDeclaration)member).getBeginLine()){
								lineEnd = ((MethodDeclaration)member).getAnnotations().get(((MethodDeclaration)member).getAnnotations().size() - 1).getEndLine();
								break;
							}
						}
						else if(member instanceof TypeDeclaration){
							lineEnd = QDoxUtil.getMethodAnnotationLineEndWithJavaParserHelper(lineStart, ((TypeDeclaration)member));
							if(lineEnd != 0){
								break outer;
							}
						}
					}
			} 

		return lineEnd;
	}

	private static int getMethodAnnotationLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(type.getMembers() != null)
			for (Iterator iterator = type.getMembers().iterator(); iterator.hasNext();) {
				BodyDeclaration member = (BodyDeclaration) iterator.next();
				if(member instanceof ConstructorDeclaration){
					if(lineStart == ((ConstructorDeclaration)member).getBeginLine()){
						lineEnd = ((ConstructorDeclaration)member).getAnnotations().get(((ConstructorDeclaration)member).getAnnotations().size() - 1).getEndLine();
						break;
					}
				}
				else if(member instanceof MethodDeclaration){
					if(lineStart == ((MethodDeclaration)member).getBeginLine()){
						lineEnd = ((MethodDeclaration)member).getAnnotations().get(((MethodDeclaration)member).getAnnotations().size() - 1).getEndLine();
						break;
					}
				}
				else if(member instanceof TypeDeclaration){
					lineEnd = QDoxUtil.getMethodAnnotationLineEndWithJavaParserHelper(lineStart, ((TypeDeclaration)member));
					if(lineEnd != 0){
						break;
					}
				}
			}

		return lineEnd;
	}

	private static int getMethodLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		
		outer:
			for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
				TypeDeclaration type = (TypeDeclaration) iterator.next();
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2
							.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof ConstructorDeclaration){
							if(lineStart == ((ConstructorDeclaration)member).getBeginLine()){
								lineEnd = ((ConstructorDeclaration)member).getEndLine();
								break;
							}
						}
						else if(member instanceof MethodDeclaration){
							if(lineStart == ((MethodDeclaration)member).getBeginLine()){
								lineEnd = ((MethodDeclaration)member).getEndLine();
								break;
							}
						}
						else if(member instanceof TypeDeclaration){
							lineEnd = QDoxUtil.getMethodLineEndWithJavaParserHelper(lineStart, ((TypeDeclaration)member));
							if(lineEnd != 0){
								break outer;
							}
						}
					}
			}

		return lineEnd;
	}

	private static int getMethodLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(type.getMembers() != null)
			for (Iterator iterator = type.getMembers().iterator(); iterator.hasNext();) {
				BodyDeclaration member = (BodyDeclaration) iterator.next();
				if(member instanceof ConstructorDeclaration){
					if(lineStart == ((ConstructorDeclaration)member).getBeginLine()){
						lineEnd = ((ConstructorDeclaration)member).getEndLine();
						break;
					}
				}
				else if(member instanceof MethodDeclaration){
					if(lineStart == ((MethodDeclaration)member).getBeginLine()){
						lineEnd = ((MethodDeclaration)member).getEndLine();
						break;
					}
				}
				else if(member instanceof TypeDeclaration){
					lineEnd = QDoxUtil.getMethodAnnotationLineEndWithJavaParserHelper(lineStart, ((TypeDeclaration)member));
					if(lineEnd != 0){
						break;
					}
				}
			}

		return lineEnd;
	}

	private static int getTypeAnnotationLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		
		for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
			TypeDeclaration type = (TypeDeclaration) iterator.next();
			if(lineStart == type.getBeginLine()){
				lineEnd = type.getAnnotations().get(((TypeDeclaration)type).getAnnotations().size() - 1).getEndLine();
				break;
			}
			if(lineEnd == 0){
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof TypeDeclaration){
							lineEnd = getTypeAnnotationLineEndWithJavaParserHelper(lineStart, type);
							if(lineEnd != 0){
								break;
							}
						}
					}
			}
		}

		return lineEnd;
	}

	private static int getTypeAnnotationLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(lineStart == type.getBeginLine()){
			lineEnd = type.getAnnotations().get(((TypeDeclaration)type).getAnnotations().size() - 1).getEndLine();
		}
		if(lineEnd == 0){
			if(type.getMembers() != null)
				for (Iterator iterator2 = type.getMembers().iterator(); iterator2.hasNext();) {
					BodyDeclaration member = (BodyDeclaration) iterator2.next();
					if(member instanceof TypeDeclaration){
						lineEnd = QDoxUtil.getTypeAnnotationLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
						if(lineEnd != 0){
							break;
						}
					}
				}
		}

		return lineEnd;
	}

	private static int getTypeLineEndWithJavaParser(int lineStart, String fileAsString) throws UnsupportedEncodingException, ParseException{
		int lineEnd = 0;
		CompilationUnit cu = JavaParser.parse(new ByteArrayInputStream(fileAsString.getBytes("UTF-8")));
		for (Iterator iterator = cu.getTypes().iterator(); iterator.hasNext();) {
			TypeDeclaration type = (TypeDeclaration) iterator.next();
			if(lineStart == type.getBeginLine()){
				lineEnd = type.getEndLine();
				break;
			}
			if(lineEnd == 0){
				if(type.getMembers() != null)
					for (Iterator iterator2 = type.getMembers().iterator(); iterator2.hasNext();) {
						BodyDeclaration member = (BodyDeclaration) iterator2.next();
						if(member instanceof TypeDeclaration){
							lineEnd = getTypeLineEndWithJavaParserHelper(lineStart, type);
							if(lineEnd != 0){
								break;
							}
						}
					}
			}
		}

		return lineEnd;
	}

	private static int getTypeLineEndWithJavaParserHelper(int lineStart, TypeDeclaration type){
		int lineEnd = 0;

		if(lineStart == type.getBeginLine()){
			lineEnd = type.getEndLine();
		}
		if(lineEnd == 0){
			if(type.getMembers() != null)
				for (Iterator iterator2 = type.getMembers().iterator(); iterator2.hasNext();) {
					BodyDeclaration member = (BodyDeclaration) iterator2.next();
					if(member instanceof TypeDeclaration){
						lineEnd = QDoxUtil.getTypeLineEndWithJavaParserHelper(lineStart, (TypeDeclaration)member);
						if(lineEnd != 0){
							break;
						}
					}
				}
		}

		return lineEnd;
	}
	
//	private static String readFile(String path, Charset encoding) 
//			throws IOException  {
//		byte[] encoded = Files.readAllBytes(Paths.get(path));
//		return encoding.decode(ByteBuffer.wrap(encoded)).toString();
//	}
}
